# -*- coding: utf-8 -*-
##############################################################################
#
#    Service module for OpenERP, Open Source Management Solutions.
#    Copyright (C) 2015 E-MIPS/Eynes (<http://www.eynes.com.ar>). All rights
#    reserved.
#
#    This file is a part of Service
#
#    Service is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    Service is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

from openerp.addons import decimal_precision as dp
from openerp.osv import fields, orm
from openerp.tools import drop_view_if_exists
from openerp.tools.translate import _


class SuitcaseCurrentMaterials(orm.Model):
    _auto = False
    _name = 'suitcase.current.materials'
    _description = 'List of current materials associated to a suitcase'

    _columns = {
        'suitcase_id': fields.many2one('stock.suitcase', 'Suitcase', readonly=True),
        'employee_id': fields.related('suitcase_id', 'employee_id', type='many2one',
                                      relation='hr.employee', string='Employee'),
        'product_id': fields.many2one('product.product', 'Material'),
        'product_qty': fields.float(digits_compute=dp.get_precision('Product Unit of Measure'),
                                    string='Quantity'),
    }

    def init(self, cr):
        """
        Use an SQL view to get the total product quantity per suitcase
        """
        drop_view_if_exists(cr, 'suitcase_current_materials')
        # We use product_id+suitcase_id as id of the view to get an unique id that will be
        # consistent through the views.
        # The SELECT statement stored as out_moves doesn't filter stock_moves only in
        # 'done' state but also in 'assigned', 'draft', and 'confirmad'. This is
        # useful when a record is deleted from the kanban view.
        cr.execute("""
                    CREATE OR REPLACE VIEW suitcase_current_materials AS
                    (
                    SELECT ROW_NUMBER() OVER (ORDER BY ipid)::INT AS id, isid suitcase_id, ipid product_id, product_qty
                    FROM (
                    SELECT in_moves.suitcase_id isid,
                             in_moves.product_id ipid, out_moves.suitcase_id osid,
                             out_moves.product_id opid, in_moves.product_qty iq,
                             out_moves.product_qty oq,
                             COALESCE(in_moves.product_qty-out_moves.product_qty,
                                     in_moves.product_qty, 0-out_moves.product_qty) product_qty
                     FROM (
                             SELECT sc.id suitcase_id, sm.product_id, sum(sm.product_qty)
                                    AS product_qty
                             FROM stock_suitcase sc
                                 JOIN stock_move sm on sm.location_dest_id=sc.location_id
                             WHERE sm.state IN ('done')
                             GROUP BY sm.product_id, sc.id
                             ) AS in_moves
                     LEFT JOIN (
                             SELECT sc.id suitcase_id, sm.product_id, sum(sm.product_qty)
                                    AS product_qty
                             FROM stock_suitcase sc
                                 JOIN stock_move sm on sm.location_id=sc.location_id
                             WHERE sm.state = 'done'
                             GROUP BY sm.product_id, sc.id
                             ) AS out_moves
                             ON in_moves.suitcase_id=out_moves.suitcase_id
                                 AND in_moves.product_id=out_moves.product_id
                     ) AS current_materials
                    WHERE product_qty > 0)
        """)

    def fields_view_get(self, cr, uid, view_id, view_type="form", context=None, toolbar=False):
        res = super(SuitcaseCurrentMaterials, self).fields_view_get(cr, uid, view_id, view_type,
                                                                    context, toolbar)
        if view_type == "search":
            cm_obj = self.pool["suitcase.current.materials"]
            cm_ids = cm_obj.search(cr, uid, [])
            all_cm = cm_obj.browse(cr, uid, cm_ids)
            dom = tuple(cm.product_id.id for cm in all_cm)
            if dom:
                res["fields"]["product_id"]["domain"].append(("id", "in", dom))
        return res

    def unlink(self, cr, uid, ids, context=None):
        if isinstance(ids, (int, long)):
            ids = [ids]

        suitcase_obj = self.pool.get("stock.suitcase")
        ml_obj = self.pool["suitcase.add.materials.line"]
        stock_loc_obj = self.pool["stock.location"]
        loc_ids = stock_loc_obj.search(cr, uid, [("scrap_location", "=", True)])
        loc_dest_id = loc_ids[0]
        for cm in self.browse(cr, uid, ids, context=context):
            line_id = ml_obj.create(cr, uid, {
                "product_id": cm.product_id.id,
                "product_uom_id": cm.product_id.uom_id.id,
                "product_uom_qty": cm.product_qty,
                "location_id": loc_dest_id,
            })
            material_lines = [ml_obj.browse(cr, uid, line_id)]
            src_doc_tmp = _("Material removed from: %(suitcase)s")
            suitcase = suitcase_obj.browse(cr, uid, cm.suitcase_id.id, context=context)
            suitcase.create_picking(sense='out', material_lines=material_lines,
                                    src_doc_tmp=src_doc_tmp)


class SuitcaseWaitingMaterials(orm.Model):
    _auto = False
    _name = 'suitcase.waiting.materials'
    _description = 'List of materials waiting to be moved to its suitcase'

    _columns = {
        'suitcase_id': fields.many2one('stock.suitcase', 'Suitcase', readonly=True),
        'product_id': fields.many2one('product.product', 'Material'),
        'product_qty': fields.float(digits_compute=dp.get_precision('Product Unit of Measure'),
                                    string='Quantity'),
    }

    def init(self, cr):
        """
        Use an SQL view to get the total product quantity per suitcase
        """
        drop_view_if_exists(cr, 'suitcase_waiting_materials')
        cr.execute("""
                   CREATE OR REPLACE VIEW suitcase_waiting_materials AS
                   (
                   SELECT in_moves.product_id+in_moves.suitcase_id id, in_moves.suitcase_id,
                          in_moves.product_id,
                          COALESCE(in_moves.product_qty-out_moves.product_qty,
                                   in_moves.product_qty, 0-out_moves.product_qty, 0) product_qty

                   FROM (
                        SELECT sc.id suitcase_id, sm.product_id, sum(sm.product_qty) product_qty
                        FROM stock_picking p
                            JOIN stock_suitcase sc on sc.location_id=p.location_dest_id
                            JOIN stock_move sm on sm.picking_id=p.id
                        WHERE sm.state IN ('draft', 'assigned', 'confirmed')
                        GROUP BY sm.product_id, sc.id
                        ) AS in_moves
                   LEFT JOIN (
                        SELECT sc.id suitcase_id, sm.product_id, sum(sm.product_qty) product_qty
                        FROM stock_picking p
                            JOIN stock_suitcase sc on sc.location_id=p.location_id
                            JOIN stock_move sm on sm.picking_id = p.id
                        WHERE sm.state IN ('draft', 'assigned', 'confirmed')
                        GROUP BY sm.product_id, sc.id
                        ) AS out_moves
                        ON in_moves.suitcase_id=out_moves.suitcase_id
                            AND in_moves.product_id=out_moves.product_id
                   )
        """)
